# -*- coding: utf-8 -*-
import os
import argparse

from pslmw_md_toc_files_plus import MdTOCFilesPlus

def set_default_value(value, ruta_base, default='default'):
    normal_path = os.path.basename(os.path.normpath(ruta_base))
    dir_path = os.path.normpath(ruta_base).replace(normal_path, "")
    new_path = dir_path + normal_path + default
    if value is not None:
        value = value[0] if value else new_path
    return value

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Generar TOC para archivos Markdown.")

    # Args obligatorio :str
    parser.add_argument(
        "input_dir",
        type=str,
        help="Ruta del directorio de entrada."
    )

    # Args opcional :boolean
    parser.add_argument(
        "-t",
        "--toc",
        action="store_true",
        help="Generar TOC del directorio de entrada."
             "Si le pasamos una ruta. Genera una copia de todos los ficheros y trabaja sobre este directorio.\n"
             "No copiará los directorios archivos ignorados."
    )
    parser.add_argument(
        "-tf",
        "--toc_files",
        action="store_true",
        help="Generar TOC por cada fichero markdown que encuentra dentro de la ruta de entrada."
    )
    parser.add_argument(
        "-rtf",
        "--rm_toc_files",
        action="store_true",
        help="Elimina TOC por cada fichero markdown que encuentra dentro de la ruta de entrada."
    )
    parser.add_argument(
        "-ts",
        "--toc_sort",
        action="store_true",
        help="Ordena los ficheros '.md' con cabecera '...(order:asc|desc)...'."
    )

    # Args opcions :list
    parser.add_argument(
        "-i",
        "--ignore",
        nargs='*',
        default=[],
        help="Lista de directorios (Sin ruta absoluta) a ignorar (separados por espacios)."
    )

    # Args opcional :(None/list).
    # Estos argumentos pueden:
    #   No ester definidos.
    #   Declararse vacios: en cual caso se le assignará un valor por defecto a posteriori.
    #   Declararse con un valor: Este vendra en formato lista, en ese caso solo usaremos el primer elemento.
    parser.add_argument(
        "-dp",
        "--destination_path",
        type=str,
        nargs='*',
        default=None,
        help="Genera una copia de todos los ficheros a la ruta destino especificada o por defector '_output' y trabaja sobre este directorio."
    )
    parser.add_argument(
        "--html",
        type=str,
        nargs='*',
        default=None,
        help="Genera una copia de todos los ficheros a la ruta destino especificada o por defector '_html' y trabaja sobre este directorio.\n"
             "Convierte los archivos .md en ficheros HTML indicando."
    )
    parser.add_argument(
        "-c",
        "--copy",
        type=str,
        nargs='*',
        default=None,
        help="Genera una copia de todos los ficheros a la ruta destino especificada o por defector '_copy'.\n"
             "No copiará los directorios archivos ignorados."
    )

    args = parser.parse_args()

    # Obtenemos los directorios raíz [OBLIGATORIO]
    input_dir = args.input_dir

    # Set los posibles argumentos que pueden no estar, estar vacios o llegar con un parametro (List) obteniendo su primer valor
    copy = set_default_value(args.copy,
                             ruta_base=input_dir,
                             default='_copy')
    destination_path = set_default_value(args.destination_path,
                                   ruta_base=input_dir,
                                   default='_output')
    html = set_default_value(args.html,
                             ruta_base=input_dir,
                             default='_html')

    # Obtenemos los parametros para las acciones a realizar. Campos boleanos
    toc = args.toc # Directorios o archivos a ignorar
    ignore = args.ignore # Directorios o archivos a ignorar
    toc_files = args.toc_files # Genera el TOC en cada archivo .md
    rm_toc_files = args.rm_toc_files # Eleminar el TOC en cada archivo .md
    toc_sort = args.toc_sort # Ordena el contenido del los archivos .md con cabecera `...(order:asc|desc)...`

    generador = MdTOCFilesPlus(input_dir)

    # Make copy.
    if copy:
        make_copy = MdTOCFilesPlus(input_dir, destination_path=copy, ignore=ignore)

    if toc_sort:
        pass

    if toc:
        # Generamos el TOC sobre la ruta base o sobre la ruta output si este parametro se ha proporcionado.
        generador.set_ignore(ignore=ignore)
        generador.set_destination_path(destination_path)
        generador.create_toc()

    if toc_files or rm_toc_files:
        if not toc:
            generador.set_ignore(ignore=ignore)
            generador.set_destination_path(destination_path)

        # Si se quiere realizar un TOC en cada uno de los ficheros .md
        generador.set_toc_files(toc_files)
        generador.set_rm_toc_files(rm_toc_files)
        generador.process_markdown_files()



# -*- coding: utf-8 -*-
import os
import shutil
import subprocess

MAIN = "md_worker/main.py"
INPUT_DIR = os.path.dirname(__file__) + "/test_default"

def rmtree(dir_path):
    try:
        shutil.rmtree(dir_path)
    except OSError as e:
        print(f"Error: {e.strerror}")

def rmfile(file_path):
    try:
        os.remove(file_path)
        print(f"File \'{file_path}\' deleted successfully.")
    except FileNotFoundError:
        print(f"File \'{file_path}\' not found.")
    except PermissionError:
        print(f"Permission denied to delete the file \'{file_path}\'.")
    except Exception as e:
        print(f"Error occurred while deleting the file: {e}")

def test_sin_argumentos():
    """Ejecuta el script sin argumentos. No se hace nada."""
    subprocess.run(["python3", MAIN, INPUT_DIR], capture_output=True, text=True)
    assert os.path.exists(INPUT_DIR)
    assert not os.path.exists(INPUT_DIR + "/TOC.md")

def test_copy():
    """Ejecuta el script con --copy con valores. Debe crear una copia del directorio de entrada al directorio de salida, ignorando los directorios y ficheros por defecto."""
    dir_path = "/tmp/copy_TOC"
    try:
        subprocess.run(["python3", MAIN, INPUT_DIR, "--copy", dir_path], capture_output=True, text=True)
        assert os.path.exists(dir_path)
        assert not os.path.exists(dir_path + "md_worker/TOC.md")
        assert not os.path.exists(dir_path + "/log.log")
    finally:
        rmtree(dir_path)

def test_copy_default():
    """Ejecuta el script con --copy sin valores. Debe crear una copia del directorio de entrada al directorio de salida por defecto, ignorando los directorios y ficheros por defecto."""
    dir_path = INPUT_DIR + "_copy"
    try:
        subprocess.run(["python3", MAIN, INPUT_DIR, "--copy"], capture_output=True, text=True)
        assert os.path.exists(dir_path)
        assert not os.path.exists(dir_path + "md_worker/TOC.md")
        assert not os.path.exists(dir_path + "/log.log")
    finally:
        rmtree(dir_path)

def test_copy_file():
    """Ejecuta el script con --copy con un valor de fichero, esperando un error."""
    result = subprocess.run(["python3", MAIN, INPUT_DIR + "/wiki.md", "--copy"], capture_output=True, text=True)
    assert "error" in result.stderr.lower()

def test_toc():
    """Ejecuta el script con --toc. Debe de generar el fichero TOC.md en la ruta base."""
    try:
        subprocess.run(["python3", MAIN, INPUT_DIR, "--toc"], capture_output=True, text=True)
        assert os.path.isfile(INPUT_DIR + "/TOC.md")
    finally:
        rmfile(INPUT_DIR+"/TOC.md")

def test_toc_path_file():
    """"Ejecuta el script con --toc de un fichero. esperando un error."""
    result = subprocess.run(["python3", MAIN, INPUT_DIR + "wiki.md", "--toc"], capture_output=True, text=True)
    assert not os.path.isfile(INPUT_DIR + "/TOC.md")
    assert "error" in result.stderr.lower()

def test_toc_output():
    """Ejecuta el script con --destination_path con un valor. Debe crear una copia del directorio de entrada al directorio de salida, ignorando los directorios y ficheros por defecto.
    Debe de genearar el fichero TOC.md a la ruta de salida"""
    destination_path = "/tmp/tests_destination_path"
    try:
        subprocess.run(["python3", MAIN, INPUT_DIR, "--toc", "--destination_path", destination_path], capture_output=True, text=True)
        assert os.path.isfile(destination_path + "/TOC.md")
    finally:
        rmtree(destination_path)

def test_toc_output_default():
    """Ejecuta el script con --destination_path sin valor. Debe crear una copia del directorio de entrada al directorio de salida por defecto, ignorando los directorios y ficheros por defecto.
    Debe de genearar el fichero TOC.md a la ruta de salida"""
    destination_path = INPUT_DIR + "_output"
    try:
        subprocess.run(["python3", MAIN, INPUT_DIR, "--toc", "--destination_path"], capture_output=True, text=True)
        assert os.path.isfile(destination_path + "/TOC.md")
    finally:
        rmtree(destination_path)

def test_toc_output_ignore():
    """Ejecuta el script con --ignaorar sin un valor. Debe crear una copia del directorio de entrada al directorio de salida, ignorando los directorios y ficheros por defecto.
    Debe de genearar el fichero TOC.md a la ruta de salida"""
    destination_path = "/tmp/test_destination_path_files"
    try:
        subprocess.run(["python3", MAIN, INPUT_DIR, "--toc", "--destination_path", destination_path, "--ignore"], capture_output=True, text=True)
        assert not os.path.exists(destination_path + "/.ignore_log.log")
        assert os.path.exists(destination_path + "/TOC.md")
    finally:
        rmtree(destination_path)

def test_toc_output_ignore_lista():
    """Ejecuta el script con --ignore con una lista de valores. Debe crear una copia del directorio de entrada al directorio de salida, ignorando los directorios y ficheros por defecto más los indicados.
    Debe de genearar el fichero TOC.md a la ruta de salida"""
    destination_path = "/tmp/test_destination_path_files"
    try:
        subprocess.run(["python3", MAIN, INPUT_DIR, "--toc", "--destination_path", destination_path, "--ignore"] + ["guies", "wiki.md"], capture_output=True, text=True)
        assert os.path.exists(destination_path + "/TOC.md")
        assert not os.path.exists(destination_path + "/guies")
        assert not os.path.isfile(destination_path + "/wiki.md")
    finally:
        rmtree(destination_path)

def test_toc_file_output():
    """Ejecuta el script con --toc_files. Debe de recorrer todos los ficheros '.md' y generar el TOC dentro de cada uno de ellos, si ya existe lo sobre escribe."""
    destination_path = "/tmp/test_destination_path_files"
    try:
        subprocess.run(["python3", MAIN, INPUT_DIR, "--destination_path", destination_path, "--toc_files"], capture_output=True, text=True)
        assert not os.path.isfile(INPUT_DIR + "/TOC.md")
        assert os.path.isfile(destination_path + "/wiki.md")
        with open(destination_path + "/wiki.md") as archivo:
            for line in archivo:
                if line == '<!-- TOC START -->':
                    assert True
    finally:
        rmtree(destination_path)

def test_toc_file_output_default():
    """Ejecuta el script con --toc_files. Debe de recorrer todos los ficheros '.md' y generar el TOC dentro de cada uno de ellos, si ya existe lo sobre escribe."""
    destination_path_files = INPUT_DIR + "_output"
    try:
        subprocess.run(["python3", MAIN, INPUT_DIR, "--destination_path", "--toc_files"], capture_output=True, text=True)
        assert not os.path.isfile(INPUT_DIR + "/TOC.md")
        assert os.path.isfile(destination_path_files + "/wiki.md")
        with open(destination_path_files + "/wiki.md") as archivo:
            for line in archivo:
                if line == '<!-- TOC FIN -->':
                    assert True
    finally:
        rmtree(destination_path_files)

def test_rm_toc_file_from_file():
    """Ejecuta el script con --rm_toc_files. Debe de recorrer todos los ficheros '.md' y eleminar el TOC dentro de cada uno de ellos."""
    with open(INPUT_DIR + "/wiki.md") as archivo:
        for line in archivo:
            if line == '<!-- TOC START -->':
                assert True
    subprocess.run(["python3", MAIN, INPUT_DIR, "--rm_toc_files"], capture_output=True, text=True)
    assert not os.path.isfile(INPUT_DIR + "/TOC.md")
    with open(INPUT_DIR + "/wiki.md") as archivo:
        for line in archivo:
            if line == '<!-- TOC START -->':
                assert False

def test_toc_file():
    """Ejecuta el script con --toc_files. Debe de recorrer todos los ficheros '.md' y generar el TOC dentro de cada uno de ellos, si ya existe lo sobre escribe."""
    subprocess.run(["python3", MAIN, INPUT_DIR, "--toc_files"], capture_output=True, text=True)
    assert not os.path.isfile(INPUT_DIR + "/TOC.md")
    with open(INPUT_DIR + "/wiki.md") as archivo:
        for line in archivo:
            if line == '<!-- TOC START -->':
                assert True

def test_rm_toc_file_destination_path():
    """Ejecuta el script con --rm_toc_files. Debe de recorrer todos los ficheros '.md' y eleminar el TOC dentro de cada uno de ellos."""
    destination_path = INPUT_DIR + "_output"
    try:
        assert not os.path.isfile(destination_path + "/TOC.md")
        with open(INPUT_DIR + "/wiki.md") as archivo:
            for line in archivo:
                if line == '<!-- TOC START -->':
                    assert True
        subprocess.run(["python3", MAIN, INPUT_DIR, "--destination_path", destination_path, "-rtf"], capture_output=True, text=True)
        assert not os.path.isfile(destination_path + "/TOC.md")
        with open(destination_path + "/wiki.md") as archivo:
            for line in archivo:
                if line == '<!-- TOC START -->':
                    assert False
        with open(INPUT_DIR + "/wiki.md") as archivo:
            for line in archivo:
                if line == '<!-- TOC START -->':
                    assert True
    finally:
        rmtree(destination_path)

def test_rm_toc_file_destination_path_file():
    """Ejecuta el script con --rm_toc_files. Debe de recorrer todos los ficheros '.md' y eleminar el TOC dentro de cada uno de ellos."""
    try:
        subprocess.run(["python3", MAIN, INPUT_DIR, "-c"], capture_output=True, text=True)
        with open(INPUT_DIR + "_copy/wiki.md") as archivo:
            for line in archivo:
                if line == '<!-- TOC START -->':
                    assert True
        subprocess.run(["python3", MAIN, INPUT_DIR + "_copy/wiki.md", "-rtf"], capture_output=True, text=True)
        with open(INPUT_DIR + "_copy/wiki.md") as archivo:
            for line in archivo:
                if line == '<!-- TOC START -->':
                    assert False
    finally:
        rmtree(INPUT_DIR + "_copy")

# TODO : SORT
# def test_sort_files_dir():
#     """Ejecuta el script con --sort. Debe de recorrer todos los ficheros '.md' en busca de la cabecera de 'ordern' y ordenar el TOC por niveles según este indicado."""
#     subprocess.run(["python3", MAIN, INPUT_DIR, "--sort"], capture_output=True, text=True)
#     rmtree(INPUT_DIR)
#     subprocess.run(["python3", MAIN, INPUT_DIR + "_base", "-c", INPUT_DIR], capture_output=True, text=True)
#
# def test_sort_files_dir_defautl():
#     """Ejecuta el script con --sort. Debe de recorrer todos los ficheros '.md' del directorio --output y buscar enla cabecera el 'ordern' y ordenar el TOC por niveles según este indicado."""
#     subprocess.run(["python3", MAIN, INPUT_DIR, "-o", "--sort"], capture_output=True, text=True)
#     rmtree(INPUT_DIR + "_output")
#
# def test_sort_file():
#     """Ejecuta el script con --sort. Debe de busca en el fichero '.md' la cabecera de 'ordern' y ordenar el TOC por niveles según este indicado."""
#     path_file = INPUT_DIR + "wiki.md"
#     subprocess.run(["python3", MAIN, path_file, "--hmtl"], capture_output=True, text=True)
#     rmfile(path_file)

# TODO : HTML
# def check_md_html(path_dir):
#     have_toc, have_ul_li_link, have_h1_internal_link = False, False, False
#     with open(path_dir + "/wiki.html") as archivo:
#         for line in archivo:
#             if line == '<!-- TOC START -->':
#                 have_toc = True
#             if line == '<li><a href="#wiki">WIKI</a></li>':
#                 have_ul_li_link = True
#             if line == '<h1 id="wiki">WIKI</h1>':
#                 have_h1_internal_link = True
#     assert have_toc and have_ul_li_link and have_h1_internal_link
#
# def test_md_html_dir():
#     path_dir = INPUT_DIR + "_md_html"
#     """Ejecuta el script con --html. Debe hacer una copia del contenido en la ruta especificada y recorrer todos los ficheros '.md' y convertir el contenido a html."""
#     subprocess.run(["python3", MAIN, INPUT_DIR, "--html", path_dir], capture_output=True, text=True)
#     assert os.path.isdir(path_dir)
#     assert os.path.isfile(path_dir + "/wiki.html")
#     check_md_html(path_dir)
#     rmtree(path_dir)
#
# def test_md_html_dir_default():
#     """Ejecuta el script con --html. Debe hacer una copia del directorio con un nombre por defecto y recorrer todos los ficheros '.md' y convertir el contenido a html."""
#     path_dir = INPUT_DIR + "_html"
#     subprocess.run(["python3", MAIN, INPUT_DIR, "--html"], capture_output=True, text=True)
#     assert os.path.isdir(path_dir)
#     assert os.path.isfile(path_dir + "/wiki.html")
#     check_md_html(path_dir)
#     rmtree(path_dir)

# def test_md_html_toc_files_output():
#     """Ejecuta el script con --html. Debe hacer una copia del directorio con un nombre por defecto y recorrer todos los ficheros '.md' y convertir el contenido a html."""
#     path_dir = INPUT_DIR + "_html"
#     subprocess.run(["python3", MAIN, INPUT_DIR, "--html", "-o", "-tf"], capture_output=True, text=True)
#     assert os.path.isdir(path_dir)
#     assert os.path.isfile(path_dir + "/wiki.html")
#     check_md_html(path_dir)
#     rmtree(path_dir)

# def test_md_html_file():
#     """Ejecuta el script con --html. Debe hacer una copia del fichero '.md' en una ruta por defecto y convertir el contenido a html."""
#     path_file = INPUT_DIR + "/wiki.md"
#     result = subprocess.run(["python3", MAIN, path_file, "--html"], capture_output=True, text=True)
#     assert "error" in result.stderr.lower()
